
/*
   Das Programm zeigt ein Beispiel dafr, wie ber HotHelp kontextsensitive
   Online-Hilfe programmiert werden kann (dieselbe Methode wurde auch in
   den HotHelpTools verwendet). Es ffnet ein Fenster mit mehreren Gadgets;
   ber eines davon kann ein Hilfs-Modus aktiviert werden, in dem das
   Anklicken eines Gadgets oder sonstigen Bereiches im Fenster einen
   Hilfstext zu dem entsprechenden Bereich darstellt (ab OS 3.00 knnte
   natrlich auch das neue GADGETHELP-Feature verwendet werden; die hier
   vorgestellte Methode hat jedoch den Vorteil, auch noch unter OS 1.3 und
   OS 2.0 zu funktionieren...).
   Bevor Sie dieses Demoprogramm testen, sollten Sie zuerst das dafr
   bentigte Beispiel-Projekt bersetzen, das die Texte fr die Online-
   Hilfe beinhaltet. Dies knnen Sie von der Shell aus z.B. ber Eingabe
   von 'HotHelpComp Bsp5Projekt G' erreichen (vorausgesetzt, Sie befinden
   sich im richtigen Verzeichnis).
*/


#include "HotHelp.h"
#include <Functions.h>
#include <StdIO.h>


/*
   Die Breite und Hhe unseres Demo-Fensters.
*/
#define  WIDTH          320
#define  HEIGHT         100


struct IntuitionBase *  IntuitionBase;
struct HotHelpBase *    HotHelpBase;
struct Window *         win;

/*
   Hierin halten wir die ID des HotHelp-Fensters, das fr Notification
   verwendete Signal und die entsprechende Signal-Maske fest. 'hh_data'
   nimmt die aktuellen Daten des HotHelp-Fensters auf und speichert sie
   zum nchsten ffnen zwischen. Die Position wird so initialisiert, da
   das HotHelp-Fenster sich beim ersten ffnen unter dem Demo-Fenster
   befindet und dieselbe Gre hat. Wird es dann verndert, geschlossen
   und nochmal geffnet, behlt es die neue Position und Gre bei.
*/
LONG                    hh_id;
LONG                    hh_signal;
LONG                    hh_sigmask;
struct HH_Data          hh_data     = { HH_VERSION, 0, 12 + HEIGHT,
                                                      WIDTH, HEIGHT };

/*
   Wir definieren ein Fenster mit einigen Gadgets, zu denen spter im
   Hilfsmodus Hilfstexte abgefragt werden knnen.
*/
WORD                    border_coords [] =
   {
   0, 11, 0, 0, 99, 0,
   99, 0, 99, 11, 0, 11,
   };
struct Border           border [] =
   {
   0, 0, 2, 0, JAM1, 3, &border_coords [0], &border [1],
   0, 0, 1, 0, JAM1, 3, &border_coords [6], NULL,
   };
struct IntuiText        itext [] =
   {
   1, 0, JAM2, 22, 2, NULL, "Gadget1", NULL,
   1, 0, JAM2, 22, 2, NULL, "Gadget2", NULL,
   1, 0, JAM2, 30, 2, NULL, "Hilfe", NULL,
   };
struct Gadget           gadg [] =
   {
   &gadg [1], 20, 20, 100, 12, GFLG_GADGHCOMP, GACT_RELVERIFY, GTYP_BOOLGADGET,
   &border [0], NULL, &itext [0], 0, NULL, 0, NULL,
   &gadg [2], 200, 20, 100, 12, GFLG_GADGHCOMP, GACT_RELVERIFY, GTYP_BOOLGADGET,
   &border [0], NULL, &itext [1], 0, NULL, 1, NULL,
   NULL, 100, 50, 100, 12, GFLG_GADGHCOMP, GACT_RELVERIFY, GTYP_BOOLGADGET,
   &border [0], NULL, &itext [2], 0, NULL, 2, NULL,
   };
struct NewWindow        nw =
   {
   0, 12, WIDTH, HEIGHT,
   -1, -1,
   IDCMP_CLOSEWINDOW    |
    IDCMP_RAWKEY        |
    IDCMP_MOUSEBUTTONS  |
    IDCMP_GADGETUP,
   WFLG_DRAGBAR         |
    WFLG_DEPTHGADGET    |
    WFLG_CLOSEGADGET    |
    WFLG_SIMPLE_REFRESH |
    WFLG_NOCAREREFRESH  |
    WFLG_ACTIVATE,
   &gadg [0], NULL,
   "Demo fr HotHelp-Onlinehilfe",
   NULL, NULL, 0, 0, 0, 0,
   WBENCHSCREEN,
   };


VOID  CheckHHWindow     (VOID);
VOID  HelpText          (BYTE *);
VOID  HelpMode          (VOID);
VOID  ProcessMessages   (VOID);


LONG HH_ShowHelpTags
   (
   struct HH_NewWindow *   hh_new,
   Tag                     tag1,
   ...
   )

   {
   return (HH_ShowHelpTagList (hh_new, (struct TagItem *) &tag1));
   }                             // HH_ShowHelpTags ()


VOID main (VOID)
   {
   if (IntuitionBase = OpenLibrary ("intuition.library", 33))
      {
      if (HotHelpBase = OpenLibrary ("hothelp.library", 3))
         {
         if ((hh_signal = AllocSignal (-1)) != -1)
            {
            hh_sigmask = 1L << hh_signal;
            if (win = OpenWindow (&nw))
               {
               ProcessMessages ();
               CloseWindow (win);
               }
            else
               printf ("Fehler bei OpenWindow\n");

            /*
               Am Programmende berprfen wir, ob das HotHelp-Fenster evtl.
               noch geffnet ist. Wenn ja, wird es hier explizit geschlossen
               (wir drfen das Fenster nicht geffnet lassen, da mit
               Notification gearbeitet wurde. Wird dann das Programm vor dem
               HotHelp-Fenster geschlossen, kann das zu ernsthaften Problemen
               fhren, wenn HotHelp dem nicht mehr existierenden Task das
               Beendigungs-Signal senden mchte. Dieses knnte sich dann
               nur allzu leicht in ein Beerdigungs-Signal verwandeln...)
            */
            CheckHHWindow ();
            if (hh_id)
               HH_ASyncCloseWindow (hh_id);

            FreeSignal (hh_signal);
            }
         else
            printf ("Fehler bei AllocSignal\n");

         CloseLibrary (HotHelpBase);
         }
      else
         printf ("Fehler bei HotHelpBase\n");
      CloseLibrary (IntuitionBase);
      }
   else
      printf ("Fehler bei IntuitionBase\n");
   }


VOID ProcessMessages (VOID)
   {
   BOOL  end   = FALSE;

   while (! end)
      {
      struct IntuiMessage *   msg;

      while (! (msg = (struct IntuiMessage *) GetMsg (win->UserPort)))
         Wait (1L << win->UserPort->mp_SigBit);

      switch (msg->Class)
         {
         case IDCMP_CLOSEWINDOW:
            end = TRUE;
            break;
         case IDCMP_RAWKEY:
            if (msg->Code == 0x45)        /* ESC   */
               end = TRUE;
            else if (msg->Code == 0x5F)   /* HELP  */
               HelpMode ();
            else
               printf ("Taste %d mit Qualifier %04x wurde gedrckt\n",
                                             msg->Code, msg->Qualifier);
            break;
         case IDCMP_GADGETUP:
            switch (((struct Gadget *) msg->IAddress)->GadgetID)
               {
               case 0:
                  printf ("Gadget1 wurde bettigt.\n");
                  break;
               case 1:
                  printf ("Gadget2 wurde bettigt.\n");
                  break;
               case 2:
                  HelpMode ();
                  break;
               }
            break;
         }

      ReplyMsg ((struct Message *) msg);
      }
   }


VOID HelpMode (VOID)
   {
   struct Requester        req;
   struct IntuiMessage *   msg;
   BOOL                    end   = FALSE;

   /*
      Wir blockieren alle Gadgets und Menus (wenn welche da wren) des
      Fensters, lassen aber MOUSEBUTTONs und RAWKEYs durch.
      Anschlieend setzen wir den Fragezeichen-Mauszeiger.
   */
   InitRequester (&req);
   req.Flags = NOISYREQ;
   Request (&req, win);
   HH_SetPointer (win, HHP_HELP);

   /*
      Der einleitende Text zu unserem Fenster wird dargestellt.
   */
   HelpText ("Demo-Fenster");

   /*
      Nun durchlaufen wir die Schleife, bis der Hilfs-Modus wieder beendet
      wird.
   */
   while (! end)
      {
      while (! (msg = (struct IntuiMessage *) GetMsg (win->UserPort))
                                                               && ! end)
         {
         LONG  signals;

         /*
            Wir warten auf ein Signal von unserem MsgPort oder von unserem
            HotHelp-Fenster. Ersteres bedeutet, da eine Nachricht vorliegt;
            letzteres zeigt an, da das HotHelp-Fenster geschlossen wurde
            und wir den Hilfs-Modus beenden sollten.
         */
         signals = Wait ((1L << win->UserPort->mp_SigBit) | hh_sigmask);

         if (signals & hh_sigmask)
            {
            end = TRUE;
            hh_id = 0;
            }
         }

      if (msg)
         {
         /*
            Die ESC-Taste schliet das HotHelp-Fenster, whrend die Help-
            Taste lediglich den Hilfsmodus beendet, das Fenster aber offen
            lt.
            Das HotHelp-Fenster wird beim ffnen nicht aktiviert, damit
            beim Aufruf mehrerer Hilfstexte nacheinander das Flackern
            der Fensterrahmen unterbleibt. Damit der Text im HotHelp-
            Fenster aber dennoch gescrollt werden kann, ohne das Fenster
            von Hand aktivieren zu mssen, leiten wir die entsprechenden
            Tasten des NumPads einfach an HotHelp weiter (ebenso knnten
            wir auch mit den Cursor-Tasten und den verschiedenen
            Qualifiern verfahren).
         */
         if (msg->Class == IDCMP_RAWKEY)
            {
            switch (msg->Code)
               {
               case 0x45:        /* ESC   */
                  HH_ASyncCloseWindow (hh_id);
               case 0x5F:        /* Help  */
                  end = TRUE;
                  break;
               case 0x1E:        /* 2 NumPad */
                  HH_ASyncScroll (hh_id, 1, 0);
                  break;
               case 0x3E:        /* 8 NumPad */
                  HH_ASyncScroll (hh_id, -1, 0);
                  break;
               case 0x3D:        /* 7 NumPad */
                  HH_ASyncPage (hh_id, -2);
                  break;
               case 0x3F:        /* 9 NumPad */
                  HH_ASyncPage (hh_id, -1);
                  break;
               case 0x1D:        /* 1 NumPad */
                  HH_ASyncPage (hh_id, 2);
                  break;
               case 0x1F:        /* 3 NumPad */
                  HH_ASyncPage (hh_id, 1);
                  break;
               }
            }
         else if (msg->Class == IDCMP_MOUSEBUTTONS)
            {
            if (msg->Code == IECODE_LBUTTON)
               {
               /*
                  Wenn wir die Nachricht erhalten, da die linke Maustaste
                  gedrckt wurde, untersuchen wir, ob sie sich dabei ber
                  einem unserer Gadgets befand (wir erhalten ja keine
                  GADGETUP-Messages, da unser Requester die Gadgets
                  blockiert). Wenn ja, wird ein zu dem Gadget gehrender
                  Text angezeigt.
                  Wurde keines der Gadgets angeklickt, wird abhngig
                  davon, ob die Maus sich in der oberen oder unteren
                  Fensterhlfte befand, ein Text angezeigt. 
                  Hier knnte dann in einem echten Programm noch berprft
                  werden, ob die Maus sich ber einem wichtigen Bereich
                  (z.B. einem Anzeigefeld) befindet, zu dem dann auch
                  ein passender Text angezeigt werden knnte.
               */
               WORD     i;
               BYTE *   text  = NULL;

               for (i = 0; i < 3 && ! text; i ++)
                  if (msg->MouseX >= gadg [i].LeftEdge                  &&
                      msg->MouseX <  gadg [i].LeftEdge + gadg [i].Width &&
                      msg->MouseY >= gadg [i].TopEdge                   &&
                      msg->MouseY <  gadg [i].TopEdge  + gadg [i].Height)
                     if (i == 0)
                        text = "Gadget1";
                     else if (i == 1)
                        text = "Gadget2";
                     else
                        text = "Hilfe";

               if (! text)
                  {
                  if (msg->MouseY < HEIGHT / 2)
                     text = "Oben";
                  else
                     text = "Unten";
                  }

               HelpText (text);
               }
            }
         ReplyMsg ((struct Message *) msg);
         }
      }

   /*
      Nun werden noch alle evtl. unbearbeiteten Messages bergangen, der
      Requester wird wieder entfernt und der Mauszeiger zurckgesetzt.
   */
   while (msg = (struct IntuiMessage *) GetMsg (win->UserPort))
      ReplyMsg ((struct Message *) msg);
   EndRequest (&req, win);
   ClearPointer (win);
   }


VOID HelpText (BYTE * key)
   {
   /*
      Diese Funktion stellt den Text 'key' aus dem Projekt 'DemoProjekt'
      dar.
      Zuerst wird berprft, ob das Fenster vielleicht noch geffnet ist.
      Wenn ja, wird dem Fenster einfach ber 'HH_ASyncStrGadget ()' der
      gewnschte Schlssel als neuer Text zugewiesen.
      Andernfalls mu das Fenster erst noch geffnet werden.
   */
   CheckHHWindow ();

   if (hh_id)
      HH_ASyncStrGadget (hh_id, "DemoProjekt", key, NULL, NULL, FALSE, 0);
   else
      {
      /*
         Als Projekt verwenden wir den Namen unseres Projektes, als
         Schlssel den bergebenen Text. Das Fenster luft asynchron
         zum Programm, wir erhalten seine Id (die zur spteren Verwendung
         in 'hh_id' festgehalten wird) und arbeiten mit Notification, um
         ein Signal zu erhalten, wenn der User das Fenster von sich aus
         schliet.
         Das Fenster wird auf demselben Screen wie unser Demo-Window
         geffnet; seine Prioritt ist um 1 hher als die des Demo-
         Programms. Das Fenster wird beim ffnen nicht aktiviert. ffnet
         hinterher ein anderer Prozess ein HotHelpWindow mit dem
         PosAbsolutLast-Flag, wird die Position unseres Fensters NICHT
         dafr verwendet (das knnen Sie testen, indem Sie nach dem ffnen
         und wieder Schlieen des Online_Hilfe-Fensters ber einen HotKey
         ein weiteres HotHelp-Fenster anfordern; dieses bernimmt die
         Position nicht von unserem Fenster). Die Option wurde verwendet,
         da das Fenster (zumindest beim Start) extrem klein ist; daher
         sollen andere Fenster sich nicht mit dieser Gre ffnen.
         Als Struktur zum Festhalten der Daten wird 'hh_data' angegeben.
      */
      HH_ShowHelpTags (NULL,
                       HHT_Project, "DemoProjekt",
                       HHT_Key, key,
                       HHT_ASyncWindow, (LONG) TRUE,
                       HHT_ID, &hh_id,
                       HHT_Notification, hh_signal,
                       HHT_ParentWindow, win,
                       HHT_Priority, (LONG) FindTask (NULL)->tc_Node.ln_Pri + 1,
                       HHT_NoActivation, (LONG) TRUE,
                       HHT_Forget, (LONG) HH_FORGET_POS,
                       HHT_DataStruct, &hh_data,
                       TAG_DONE);
      }
   }


VOID CheckHHWindow
   (
   VOID
   )

   {
   /*
      Wir berprfen ber diese Funktion, ob HotHelp uns inzwischen das
      Schlie-Signal gesendet hat. Wenn wir das Signal erkennen, lschen
      wir es ber 'Wait ()' und setzen die ID auf 0 zurck.
   */
   if (SetSignal (0, 0) & hh_sigmask)
      {
      Wait (hh_sigmask);
      hh_id = 0;
      }
   }                             // CheckHHWindow ()


