
/*
   Jetzt wird es ernst: Dieses Programm stellt einen Handler vor, der ein
   eigenes Pseudo-Projekt bereitstellt. Dieses Projekt namens 'SysInfo'
   enthlt jedoch keine statischen Texte; statt dessen knnen hier ber
   die diversen Schlssel aktuelle Informationen ber das System abgefragt
   werden.
*/

#include "HotHelp.h"
#include	<Exec/Memory.h>
#include	<Exec/ExecBase.h>
#include <Intuition/IntuitionBase.h>
#include <Functions.h>
#include <StdIO.h>
#include <String.h>

LONG        GetKeyInfo     (LONG, struct HH_HandlerData *);
LONG        TranslateBuf   (struct HH_HandlerData *);
LONG        ShowStartseite (struct HH_HandlerData *);
LONG        HotHelpHandler (struct HH_HandlerData *);
#pragma     regcall        (HotHelpHandler (A0))
VOID        int_start      (VOID);
VOID        int_end        (VOID);


extern struct ExecBase *   SysBase;
extern struct DosLibrary * DOSBase;

struct IntuitionBase *     IntuitionBase;
struct HotHelpBase *       HotHelpBase;
struct Interrupt           inter;
struct HH_TranslateData    hh_trans;
struct HH_CustomText       hh_ctext;
BYTE                       buf [1000];


/*
   Der Name unseres Pseudo-Projektes und die Namen der Schlssel, die
   es beinhaltet sowie deren Anzahl und die Indizes in das Feld.
   Die Liste der Schlssel sollte alphabetisch sortiert sein, damit die
   evtl. erzeugten bersichten auch wie blich sortiert sind.
*/
#define  PRO_NAME          "SysInfo"
#define  NUMB_KEYS         6
BYTE *                     keys [NUMB_KEYS] =
   {
   "Datum",
   "OS-Version",
   "Prozessor",
   "Screens",
   "Startseite",
   "Windows",
   };
#define  DATUM             0
#define  OSVERSION         1
#define  PROCESSOR         2
#define  SCREENS           3
#define  STARTSEITE        4
#define  WINDOWS           5

VOID main (VOID)
   {
   if (IntuitionBase = OpenLibrary ("intuition.library", 33))
      {
      if (HotHelpBase = OpenLibrary ("hothelp.library", 3))
         {
         /*
            Der Handler wird mit Prioritt 0 installiert (dieser Wert
            sollte grundstzlich verwendet werden, wenn keine besonderen
            Grnde fr die Verwendung eines hheren oder niedrigeren
            Wertes existieren, wie dies in Beispiel6 der Fall war).
         */
         inter.is_Code        = HotHelpHandler;
         inter.is_Data        = NULL;
         inter.is_Node.ln_Pri = 0;

         HH_AddHandler (&inter);
         Wait (SIGBREAKF_CTRL_C | SIGBREAKF_CTRL_D |
               SIGBREAKF_CTRL_E | SIGBREAKF_CTRL_F);
         HH_RemHandler (&inter);

         CloseLibrary (HotHelpBase);
         }
      else
         printf ("Fehler bei HotHelpBase\n");
      CloseLibrary (IntuitionBase);
      }
   else
      printf ("Fehler bei IntuitionBase\n");
   }


LONG HotHelpHandler
   (
   struct HH_HandlerData * hdata
   )

   {
   LONG                    ret   = FALSE;

   int_start ();

   switch (hdata->action)
      {
      case HHA_PROJECTS:
         {
         struct Node *  new;
         /*
            Dieses Kommando erhalten wir, wenn HotHelp eine bersicht
            aller von Handlern gebildeter Projekte anfordert. Wir erzeugen
            dynamisch eine Node-Struktur. Tritt ein Fehler auf, geben
            wir -1 zurck; andernfalls wird der Name unseres Projektes
            in dem neuen Node festgehalten und er wird in die entsprechende
            Liste eingetragen.
         */
         if (new = AllocMem (sizeof (struct Node), MEMF_CLEAR))
            {
            new->ln_Name = PRO_NAME;
            AddHead (&hdata->pro_names, new);
            ret = TRUE;
            }
         else
            ret = -1;
         }
         break;
      case HHA_KEY:
         /*
            Als erstes berprfen wird, ob der geforderte Projekt-Name
            mit dem Namen unseres Projektes bereinstimmt. Wenn nicht,
            brauchen wir uns nicht weiter um die Anforderung zu kmmern
            und geben FALSE zurck.
         */
         if (HH_CheckPattern (hdata->project, PRO_NAME))
            {
            LONG  n_keys   = 0;
            LONG  index;
            LONG  i;

            /*
               Das SysInfo-Projekt wurde angesprochen. Jetzt mu noch
               unterschieden werden, ob nur ein einzelner Schlssel das
               Ziel der Suche ist (dann wird die entsprechende Aktion
               durchgefhrt) oder ob ein Muster eingegeben wurde, zu dem
               mehrere Schlssel passen (dann erzeugen wir eine bersicht
               dieser Schlssel). Finden wir keinen passenden Schlssel,
               geben wir einfach FALSE zurck.
               In 'index' halten wir den Index des zuletzt gefundenen
               Schlssels fest.
            */
            for (i = 0; i < NUMB_KEYS; i ++)
               if (HH_CheckPattern (hdata->key, keys [i]))
                  {
                  index = i;
                  n_keys ++;
                  }

            if (n_keys == 1)
               ret = GetKeyInfo (index, hdata);
            else if (n_keys > 1)
               {
               sprintf (buf, "\n|SCHR-1Projekt %s\n\n", PRO_NAME);

               /*
                  Das liee sich natrlich noch optimieren, wenn wir oben
                  das Ergebnis der einzelnen Vergleiche festhalten wrden...
                  aber es ist ja nur ein kleines Beispiel...
               */
               for (i = 0; i < NUMB_KEYS; i ++)
                  if (HH_CheckPattern (hdata->key, keys [i]))
                     {
                     strcat (buf, "|QV|");
                     strcat (buf, keys [i]);
                     strcat (buf, "|QV|\t");
                     }
               ret = TranslateBuf (hdata);
               }
            }
         break;
      case HHA_PROJECT_START:
         /*
            Nur wenn die Startseite unseres Projektes angefordert wird,
            drfen wir sie auch zurckgeben.
         */
         if (! HH_StrCmp (hdata->project, PRO_NAME))
            ret = ShowStartseite (hdata);
         break;
      case HHA_FREE:
         HH_TranslateFree (&hh_trans);
         break;
      }

   int_end ();

   return (ret);
   }


/*
   Die Funktion bersetzt den Inhalt des globalen Strings 'buf' und
   trgt das Ergebnis der bersetzung in die bergebene HH_HandlerData-
   Struktur ein.
*/
LONG TranslateBuf
   (
   struct HH_HandlerData * hdata
   )

   {
   LONG                    ret;

   /*
      Wir geben auch noch eine Querverweis-Marke an, da bei den
      Schlssel-bersichten hauptschlich Querverweise angezeigt werden.
      Die Tabulator-Schrittweite setzen wir auf 30, damit die
      bersichten wie die Standard-bersichten aussehen.
   */
   hh_trans.version        = HH_VERSION;
   hh_trans.ascii_text     = buf;
   hh_trans.ascii_text_len = strlen (buf);
   hh_trans.mark_ref       = "|QV|";
   hh_trans.mark_style     = "|SCHR-";
   hh_trans.mark_para      = "|ABS|";
   hh_trans.tab_size       = 30;
   hh_trans.compress       = FALSE;
   hh_ctext.version        = HH_VERSION;

   if (HH_Translate (&hh_trans, &hh_ctext))
      {
      ret = TRUE;
      hdata->custom_text = &hh_ctext;
      }
   else if (hh_trans.error == HHT_ERROR_MEM)
      ret = -1;
   else
      ret = -3;

   return (ret);
   }


/*
   Die Funktion erzeugt die Startseite unseres Projektes, die lediglich
   eine bersicht ber alle Schlssel in Form von Querverweisen
   beinhaltet. Wir bentigen die Funktion, wenn HotHelp explizit unsere
   Startseite anfordert oder wenn der Schlssel 'Startseite' angezeigt
   werden soll. Dies ist auch der einzige Schlssel, der in der bersicht
   nicht angezeigt wird, da er lediglich auf sich selber zurckfhrt.
*/
LONG ShowStartseite
   (
   struct HH_HandlerData * hdata
   )

   {
   LONG                    i;

   sprintf (buf, "\n|SCHR-1Projekt %s\n\n", PRO_NAME);

   for (i = 0; i < NUMB_KEYS; i ++)
      if (i != STARTSEITE)
         {
         strcat (buf, "|QV|");
         strcat (buf, keys [i]);
         strcat (buf, "|QV|\t");
         }

   return (TranslateBuf (hdata));
   }


/*
   Die Funktion wird aufgerufen, wenn einer der Schlssel unseres
   Projektes ausgewhlt wurde. Sie fllt den globalen String 'buf' mit
   einem entsprechenden Text, lt ihn bersetzen und gibt einen
   entsprechenden Wert zurck.
*/
LONG GetKeyInfo
   (
   LONG                    index,
   struct HH_HandlerData * hdata
   )

   {
   LONG                    ret;

   switch (index)
      {
      case STARTSEITE:
         ret = ShowStartseite (hdata);
         break;
      case SCREENS:
         {
         LONG              ilock = LockIBase (0);
         struct Screen *   scr   = IntuitionBase->FirstScreen;

         strcpy (buf, "\nDie folgenden Bildschirme sind momentan "
                                                   "geffnet:\n\n");

         while (scr)
            {
            strcat (buf, (BYTE *) scr->Title);
            strcat (buf, "|ABS|\n");
            scr = scr->NextScreen;
            }
         UnlockIBase (ilock);

         ret = TranslateBuf (hdata);
         }
         break;
      case WINDOWS:
         {
         LONG              ilock = LockIBase (0);
         struct Screen *   scr   = IntuitionBase->FirstScreen;

         strcpy (buf, "\nDie folgenden Fenster und Bildschirme wurden "
                                                      "gefunden:\n\n");

         while (scr)
            {
            struct Window *   win   = scr->FirstWindow;

            strcat (buf, "|SCHR-2");
            strcat (buf, (scr->Title ? (BYTE *) scr->Title :
                                       "*** Namenlos ***"));
            strcat (buf, "|ABS|\n");

            while (win)
               {
               strcat (buf, "   ");
               strcat (buf, (win->Title ? win->Title :
                                 "*** Namenlos ***"));
               strcat (buf, "|ABS|\n");
               win = win->NextWindow;
               }

            scr = scr->NextScreen;
            }
         UnlockIBase (ilock);

         ret = TranslateBuf (hdata);
         }
         break;
      case OSVERSION:
         if (SysBase->LibNode.lib_Version < 36)
            strcpy (buf, "\nSie arbeiten noch mit OS 1.3 "
                         "(warum eigentlich?)");
         else if (SysBase->LibNode.lib_Version == 36)
            strcpy (buf, "\nSie arbeiten noch mit der 36'er Version "
                         "von OS 2.0? Eiwei, das erfordert Mut...");
         else if (SysBase->LibNode.lib_Version == 38)
            strcpy (buf, "\nSie arbeiten mit OS 2.1 - gar nicht "
                                                      "schlecht...");
         else if (SysBase->LibNode.lib_Version == 39)
            strcpy (buf, "\nWas denn? Sie arbeiten schon mit OS 3.0? Sie "
                         "werden doch wohl nicht etwa sogar einen A1200 "
                         "oder A4000 mit dem schnen neuen AA-ChipSet "
                         "Ihr Eigen nennen? (Gier, Neid) Oder ist das "
                         "hier blo die 'Developer'-Version von 3.0?");
         else if (SysBase->LibNode.lib_Version > 39)
            strcpy (buf, "\nDa ist wohl auch dieses Demo-Programm von "
                         "der Entwicklung berholt worden... Diese "
                         "OS-Version kennt es nmlich noch nicht!");

         ret = TranslateBuf (hdata);
         break;
      case PROCESSOR:
         strcpy (buf, "\nSie besitzen einen Amiga mit 680");
         if (SysBase->AttnFlags & AFF_68040)
            strcat (buf, "4");
         else if (SysBase->AttnFlags & AFF_68030)
            strcat (buf, "3");
         else if (SysBase->AttnFlags & AFF_68020)
            strcat (buf, "2");
         else if (SysBase->AttnFlags & AFF_68010)
            strcat (buf, "1");
         else
            strcat (buf, "0");
         strcat (buf, "0-Prozessor!");

         ret = TranslateBuf (hdata);
         break;
      case DATUM:
         /*
            Wenn Sie nicht ber die OS-2.0-Includes verfgen, sollten Sie
            den folgenden Teil auf Kommentar setzen...
         */
         if (DOSBase->dl_lib.lib_Version < 36)
            strcpy (buf, "\nSorry, aber diese Funktion bentigt "
                         "mindestens OS 2.0!");
         else
            {
            struct DateTime   dt;
            BYTE              day  [LEN_DATSTRING];
            BYTE              date [LEN_DATSTRING];
            BYTE              time [LEN_DATSTRING];

            memset (&dt, 0, sizeof (struct DateTime));
            DateStamp (&dt.dat_Stamp);
            dt.dat_StrDay  = day;
            dt.dat_StrDate = date;
            dt.dat_StrTime = time;

            if (DateToStr (&dt))
               sprintf (buf, "\nWir haben %s Uhr am einem %s, den %s",
                                                      time, day, date);
            else
               strcpy (buf, "\n'DateToStr ()' hat versagt...");
            }
         ret = TranslateBuf (hdata);
         break;
      }

   return (ret);
   }


